@@ -1,30 +1,30 @@
 /*
  *   Copyright 2013 Martin Gräßlin <mgraesslin@kde.org>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
+ *   This library is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *   Lesser General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 #include "nettesthelper.h"
 #include <netwm.h>
 #include <qtest_widgets.h>
 #include <QProcess>
+// system
+#include <unistd.h>
 
 class Property : public QScopedPointer<xcb_get_property_reply_t, QScopedPointerPodDeleter>
 {
 public:
-    Property(xcb_get_property_reply_t *p = 0) : QScopedPointer<xcb_get_property_reply_t, QScopedPointerPodDeleter>(p) {}
+    Property(xcb_get_property_reply_t *p = nullptr) : QScopedPointer<xcb_get_property_reply_t, QScopedPointerPodDeleter>(p) {}
 };
 
 Q_DECLARE_METATYPE(NET::Orientation)
@@ -57,6 +57,7 @@ private Q_SLOTS:
     void testWorkArea();
     void testActiveWindow();
     void testVirtualRoots();
+    void testDontCrashMapViewports();
 
 private:
     void waitForPropertyChange(NETRootInfo *info, xcb_atom_t atom, NET::Property prop, NET::Property2 prop2 = NET::Property2(0));
@@ -86,23 +87,33 @@ void NetRootInfoTestWM::initTestCase()
 void NetRootInfoTestWM::init()
 {
     // first reset just to be sure
-    m_connection = Q_NULLPTR;
+    m_connection = nullptr;
     m_supportWindow = XCB_WINDOW_NONE;
     // start Xvfb
     m_xvfb.reset(new QProcess);
-    // randomize the display id in [1, 98]
-    // 0 is not used because it conflicts with "normal" X server
-    // 99 is not used because it's used by KDE's CI infrastructure
-    const QString id = QStringLiteral(":") + QString::number((qrand() % 98) + 1);
-    m_xvfb->start(QStringLiteral("Xvfb"), QStringList() << id);
+    // use pipe to pass fd to Xvfb to get back the display id
+    int pipeFds[2];
+    QVERIFY(pipe(pipeFds) == 0);
+    m_xvfb->start(QStringLiteral("Xvfb"), QStringList{ QStringLiteral("-displayfd"), QString::number(pipeFds[1]) });
     QVERIFY(m_xvfb->waitForStarted());
     QCOMPARE(m_xvfb->state(), QProcess::Running);
-    // give it some time before we open the X Display
-    QTest::qWait(100);
+
+    // reads from pipe, closes write side
+    close(pipeFds[1]);
+
+    QFile readPipe;
+    QVERIFY(readPipe.open(pipeFds[0], QIODevice::ReadOnly, QFileDevice::AutoCloseHandle));
+    QByteArray displayNumber = readPipe.readLine();
+    readPipe.close();
+
+    displayNumber.prepend(QByteArray(":"));
+    displayNumber.remove(displayNumber.size() -1, 1);
+
     // create X connection
     int screen = 0;
-    m_connection = xcb_connect(qPrintable(id), &screen);
+    m_connection = xcb_connect(displayNumber.constData(), &screen);
     QVERIFY(m_connection);
+    QVERIFY(!xcb_connection_has_error(m_connection));
     m_rootWindow = KXUtils::rootWindow(m_connection, screen);
     uint32_t values[] = {XCB_EVENT_MASK_PROPERTY_CHANGE};
     xcb_change_window_attributes(m_connection, m_rootWindow,
@@ -232,6 +243,8 @@ void NetRootInfoTestWM::testSupported()
     count += 1;
     // XAWState, WMGeometry, WM2TransientFor, WM2GroupLeader, WM2WindowClass, WM2WindowRole, WM2ClientMachine
     count -= 7;
+    // WM2BlockCompositing has two properties
+    count +=1;
 
     QVERIFY(supported  != XCB_ATOM_NONE);
     QVERIFY(utf8String != XCB_ATOM_NONE);
@@ -252,24 +265,24 @@ void NetRootInfoTestWM::testSupported()
     xcb_get_property_cookie_t  wmNameCookie = xcb_get_property_unchecked(connection(), false, m_supportWindow,
             wmName, utf8String, 0, 16);
 
-    Property supportedReply(xcb_get_property_reply(connection(), supportedCookie, Q_NULLPTR));
+    Property supportedReply(xcb_get_property_reply(connection(), supportedCookie, nullptr));
     QVERIFY(!supportedReply.isNull());
     QCOMPARE(supportedReply->format, uint8_t(32));
     QCOMPARE(supportedReply->value_len, uint32_t(count));
     // TODO: check that the correct atoms are set?
-    Property wmCheckRootReply(xcb_get_property_reply(connection(), wmCheckRootCookie, Q_NULLPTR));
+    Property wmCheckRootReply(xcb_get_property_reply(connection(), wmCheckRootCookie, nullptr));
     QVERIFY(!wmCheckRootReply.isNull());
     QCOMPARE(wmCheckRootReply->format, uint8_t(32));
     QCOMPARE(wmCheckRootReply->value_len, uint32_t(1));
     QCOMPARE(reinterpret_cast<xcb_window_t *>(xcb_get_property_value(wmCheckRootReply.data()))[0], m_supportWindow);
 
-    Property wmCheckSupportReply(xcb_get_property_reply(connection(), wmCheckSupportWinCookie, Q_NULLPTR));
+    Property wmCheckSupportReply(xcb_get_property_reply(connection(), wmCheckSupportWinCookie, nullptr));
     QVERIFY(!wmCheckSupportReply.isNull());
     QCOMPARE(wmCheckSupportReply->format, uint8_t(32));
     QCOMPARE(wmCheckSupportReply->value_len, uint32_t(1));
     QCOMPARE(reinterpret_cast<xcb_window_t *>(xcb_get_property_value(wmCheckSupportReply.data()))[0], m_supportWindow);
 
-    Property wmNameReply(xcb_get_property_reply(connection(), wmNameCookie, Q_NULLPTR));
+    Property wmNameReply(xcb_get_property_reply(connection(), wmNameCookie, nullptr));
     QVERIFY(!wmNameReply.isNull());
     QCOMPARE(wmNameReply->format, uint8_t(8));
     QCOMPARE(wmNameReply->value_len, uint32_t(15));
@@ -297,7 +310,7 @@ void NetRootInfoTestWM::testSupported()
     // lets get supported again
     supportedCookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                       supported, XCB_ATOM_ATOM, 0, 90);
-    supportedReply.reset(xcb_get_property_reply(connection(), supportedCookie, Q_NULLPTR));
+    supportedReply.reset(xcb_get_property_reply(connection(), supportedCookie, nullptr));
     QVERIFY(!supportedReply.isNull());
     QCOMPARE(supportedReply->format, uint8_t(32));
     QCOMPARE(supportedReply->value_len, uint32_t(count - 7));
@@ -313,10 +326,10 @@ void NetRootInfoTestWM::testSupported()
     // lets get supported again
     supportedCookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                       supported, XCB_ATOM_ATOM, 0, 90);
-    supportedReply.reset(xcb_get_property_reply(connection(), supportedCookie, Q_NULLPTR));
+    supportedReply.reset(xcb_get_property_reply(connection(), supportedCookie, nullptr));
     QVERIFY(!supportedReply.isNull());
     QCOMPARE(supportedReply->format, uint8_t(32));
-    QCOMPARE(supportedReply->value_len, uint32_t(count - 8));
+    QCOMPARE(supportedReply->value_len, uint32_t(count - 9));
     NETRootInfo clientInfo(connection(), NET::Supported | NET::SupportingWMCheck);
     waitForPropertyChange(&clientInfo, supported, NET::Supported);
     waitForPropertyChange(&clientInfo, wmCheck, NET::SupportingWMCheck);
@@ -348,7 +361,7 @@ void NetRootInfoTestWM::testClientList()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_WINDOW, 0, 5);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(5));
@@ -393,7 +406,7 @@ void NetRootInfoTestWM::testClientListStacking()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_WINDOW, 0, 5);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(5));
@@ -437,7 +450,7 @@ void NetRootInfoTestWM::testVirtualRoots()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_WINDOW, 0, 5);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(5));
@@ -470,7 +483,7 @@ void NetRootInfoTestWM::testNumberOfDesktops()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_CARDINAL, 0, 1);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(1));
@@ -496,7 +509,7 @@ void NetRootInfoTestWM::testCurrentDesktop()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_CARDINAL, 0, 1);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(1));
@@ -529,7 +542,7 @@ void NetRootInfoTestWM::testDesktopNames()
     QVERIFY(utf8String != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, utf8String, 0, 10000);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(8));
     QCOMPARE(reply->value_len, uint32_t(8));
@@ -561,7 +574,7 @@ void NetRootInfoTestWM::testActiveWindow()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_WINDOW, 0, 1);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(1));
@@ -592,7 +605,7 @@ void NetRootInfoTestWM::testDesktopGeometry()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_CARDINAL, 0, 2);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(2));
@@ -641,7 +654,7 @@ void NetRootInfoTestWM::testDesktopLayout()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_CARDINAL, 0, 4);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(4));
@@ -695,7 +708,7 @@ void NetRootInfoTestWM::testDesktopViewports()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_CARDINAL, 0, 8);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(8));
@@ -746,7 +759,7 @@ void NetRootInfoTestWM::testShowingDesktop()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_CARDINAL, 0, 1);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(1));
@@ -803,7 +816,7 @@ void NetRootInfoTestWM::testWorkArea()
     QVERIFY(atom != XCB_ATOM_NONE);
     xcb_get_property_cookie_t cookie = xcb_get_property_unchecked(connection(), false, rootInfo.rootWindow(),
                                        atom, XCB_ATOM_CARDINAL, 0, 16);
-    Property reply(xcb_get_property_reply(connection(), cookie, Q_NULLPTR));
+    Property reply(xcb_get_property_reply(connection(), cookie, nullptr));
     QVERIFY(!reply.isNull());
     QCOMPARE(reply->format, uint8_t(32));
     QCOMPARE(reply->value_len, uint32_t(16));
@@ -845,6 +858,15 @@ void NetRootInfoTestWM::testWorkArea()
     QCOMPARE(compareThree2.size.height, 0);
 }
 
+void NetRootInfoTestWM::testDontCrashMapViewports()
+{
+    QProcess p;
+    p.start(QLatin1String(AUTOTEST_BUILD_DIR) + QLatin1String("/dontcrashmapviewport"), QStringList());
+    QVERIFY(p.waitForFinished());
+    QCOMPARE(p.exitStatus(), QProcess::NormalExit);
+    QCOMPARE(p.exitCode(), 0);
+}
+
 QTEST_GUILESS_MAIN(NetRootInfoTestWM)
 
 #include "netrootinfotestwm.moc"
